@@ -1,17 +1,14 @@
+# -*- coding: utf-8 -*-
+
+from functools import partial
+from io import StringIO
 import os
 import re
 
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
-
-from functools import partial
-
-from six import string_types, b
+import six
 from pexpect import TIMEOUT, EOF
 
-from pytest import raises
+import pytest
 from mock import patch
 from mock import Mock
 
@@ -19,7 +16,6 @@ from ansible_runner.runner_config import RunnerConfig, ExecutionMode
 from ansible_runner.loader import ArtifactLoader
 from ansible_runner.exceptions import ConfigurationError
 
-
 try:
     Pattern = re._pattern_type
 except AttributeError:
@@ -49,7 +45,7 @@ def test_runner_config_init_defaults():
 
 def test_runner_config_with_artifact_dir():
     rc = RunnerConfig('/', artifact_dir='/this-is-some-dir')
-    assert rc.artifact_dir == os.path.join('/this-is-some-dir', 'artifacts/%s' % rc.ident)
+    assert rc.artifact_dir == os.path.join('/this-is-some-dir', rc.ident)
 
 
 def test_runner_config_init_with_ident():
@@ -73,7 +69,7 @@ def test_runner_config_project_dir():
 
 
 def test_prepare_environment_vars_only_strings():
-    rc = RunnerConfig(private_data_dir="/")
+    rc = RunnerConfig(private_data_dir="/", envvars=dict(D='D'))
 
     value = dict(A=1, B=True, C="foo")
     envvar_side_effect = partial(load_file_side_effect, 'env/envvars', value)
@@ -81,11 +77,13 @@ def test_prepare_environment_vars_only_strings():
     with patch.object(rc.loader, 'load_file', side_effect=envvar_side_effect):
         rc.prepare_env()
         assert 'A' in rc.env
-        assert isinstance(rc.env['A'], string_types)
+        assert isinstance(rc.env['A'], six.string_types)
         assert 'B' in rc.env
-        assert isinstance(rc.env['B'], string_types)
+        assert isinstance(rc.env['B'], six.string_types)
         assert 'C' in rc.env
-        assert isinstance(rc.env['C'], string_types)
+        assert isinstance(rc.env['C'], six.string_types)
+        assert 'D' in rc.env
+        assert rc.env['D'] == 'D'
 
 
 def test_prepare_env_ad_hoc_command():
@@ -185,7 +183,8 @@ def test_prepare_env_directory_isolation():
         assert rc.cwd == '/tmp/foo'
 
 
-def test_prepare_inventory():
+@patch('os.path.exists', return_value=True)
+def test_prepare_inventory(path_exists):
     rc = RunnerConfig(private_data_dir='/')
     rc.prepare_inventory()
     assert rc.inventory == '/inventory'
@@ -195,11 +194,17 @@ def test_prepare_inventory():
     rc.inventory = 'localhost,anotherhost,'
     rc.prepare_inventory()
     assert rc.inventory == 'localhost,anotherhost,'
+    path_exists.return_value = False
+    rc.inventory = None
+    rc.prepare_inventory()
+    assert rc.inventory is None
 
 
 def test_generate_ansible_command():
     rc = RunnerConfig(private_data_dir='/', playbook='main.yaml')
-    rc.prepare_inventory()
+    with patch('os.path.exists') as path_exists:
+        path_exists.return_value=True
+        rc.prepare_inventory()
     rc.extra_vars = None
 
     cmd = rc.generate_ansible_command()
@@ -207,11 +212,14 @@ def test_generate_ansible_command():
 
     rc.extra_vars = dict(test="key")
     cmd = rc.generate_ansible_command()
-    assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '\'test="key"\'', 'main.yaml']
+    assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '{"test":"key"}', 'main.yaml']
 
     with patch.object(rc.loader, 'isfile', side_effect=lambda x: True):
         cmd = rc.generate_ansible_command()
-        assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', '-e', '\'test="key"\'', 'main.yaml']
+        assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', '-e', '{"test":"key"}', 'main.yaml']
+        rc.extra_vars = '/tmp/extravars.yml'
+        cmd = rc.generate_ansible_command()
+        assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', '-e', '@/tmp/extravars.yml', 'main.yaml']
         rc.extra_vars = None
         cmd = rc.generate_ansible_command()
         assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '@/env/extravars', 'main.yaml']
@@ -230,8 +238,14 @@ def test_generate_ansible_command():
     assert cmd == ['ansible-playbook', 'main.yaml']
     rc.inventory = None
 
+    with patch('os.path.exists', return_value=False) as path_exists:
+        rc.prepare_inventory()
+        cmd = rc.generate_ansible_command()
+    assert cmd == ['ansible-playbook', 'main.yaml']
+
     rc.verbosity = 3
-    rc.prepare_inventory()
+    with patch('os.path.exists', return_value=True) as path_exists:
+        rc.prepare_inventory()
     cmd = rc.generate_ansible_command()
     assert cmd == ['ansible-playbook', '-i', '/inventory', '-vvv', 'main.yaml']
     rc.verbosity = None
@@ -251,26 +265,38 @@ def test_generate_ansible_command():
     cmd = rc.generate_ansible_command()
     assert cmd == ['ansible', '-i', '/inventory', '-m', 'setup', '-a', 'test=string']
     rc.module_args = None
+    rc.module = None
+
+    rc.forks = 5
+    cmd = rc.generate_ansible_command()
+    assert cmd == ['ansible-playbook', '-i', '/inventory', '--forks', '5', 'main.yaml']
 
 
 def test_generate_ansible_command_with_api_extravars():
     rc = RunnerConfig(private_data_dir='/', playbook='main.yaml', extravars={"foo":"bar"})
-    rc.prepare_inventory()
+    with patch('os.path.exists') as path_exists:
+        path_exists.return_value=True
+        rc.prepare_inventory()
 
     cmd = rc.generate_ansible_command()
-    assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '\'foo="bar"\'', 'main.yaml']
+    assert cmd == ['ansible-playbook', '-i', '/inventory', '-e', '{"foo":"bar"}', 'main.yaml']
 
 
-def test_generate_ansible_command_with_cmdline_args():
+@pytest.mark.parametrize('cmdline,tokens', [
+    (u'--tags foo --skip-tags', ['--tags', 'foo', '--skip-tags']),
+    (u'--limit "䉪ቒ칸ⱷ?噂폄蔆㪗輥"', ['--limit', '䉪ቒ칸ⱷ?噂폄蔆㪗輥']),
+])
+def test_generate_ansible_command_with_cmdline_args(cmdline, tokens):
     rc = RunnerConfig(private_data_dir='/', playbook='main.yaml')
-    rc.prepare_inventory()
+    with patch('os.path.exists') as path_exists:
+        path_exists.return_value = True
+        rc.prepare_inventory()
     rc.extra_vars = {}
 
-    cmdline_side_effect = partial(load_file_side_effect, 'env/cmdline', b('--tags foo --skip-tags'))
-
+    cmdline_side_effect = partial(load_file_side_effect, 'env/cmdline', cmdline)
     with patch.object(rc.loader, 'load_file', side_effect=cmdline_side_effect):
         cmd = rc.generate_ansible_command()
-        assert cmd == ['ansible-playbook', '--tags', 'foo', '--skip-tags', '-i', '/inventory', 'main.yaml']
+        assert cmd == ['ansible-playbook'] + tokens + ['-i', '/inventory', 'main.yaml']
 
 
 def test_prepare_command_defaults():
@@ -279,7 +305,7 @@ def test_prepare_command_defaults():
     cmd_side_effect = partial(load_file_side_effect, 'args')
 
     def generate_side_effect():
-        return StringIO('test "string with spaces"')
+        return StringIO(u'test "string with spaces"')
 
     with patch.object(rc.loader, 'load_file', side_effect=cmd_side_effect):
         with patch.object(rc, 'generate_ansible_command', side_effect=generate_side_effect):
@@ -298,7 +324,7 @@ def test_prepare_with_defaults():
     rc.artifact_dir = '/'
     rc.env = {}
 
-    with raises(ConfigurationError) as exc:
+    with pytest.raises(ConfigurationError) as exc:
         rc.prepare()
 
     assert str(exc.value) == 'No executable for runner to run'
@@ -394,13 +420,22 @@ def test_process_isolation_defaults():
     rc.playbook = 'main.yaml'
     rc.command = 'ansible-playbook'
     rc.process_isolation = True
-    rc.prepare()
+    with patch('os.path.exists') as path_exists:
+        path_exists.return_value=True
+        rc.prepare()
 
     assert rc.command == [
         'bwrap',
+        '--die-with-parent',
         '--unshare-pid',
-        '--dev-bind', '/', '/',
+        '--dev', '/dev',
         '--proc', '/proc',
+        '--dir', '/tmp',
+        '--ro-bind', '/bin', '/bin',
+        '--ro-bind', '/etc', '/etc',
+        '--ro-bind', '/usr', '/usr',
+        '--ro-bind', '/opt', '/opt',
+        '--symlink', 'usr/lib64', '/lib64',
         '--bind', '/', '/',
         '--chdir', '/project',
         'ansible-playbook', '-i', '/inventory', 'main.yaml',
@@ -411,22 +446,36 @@ def test_process_isolation_defaults():
 @patch('shutil.copytree', return_value=True)
 @patch('tempfile.mkdtemp', return_value="/tmp/dirisolation/foo")
 @patch('os.chmod', return_value=True)
-def test_process_isolation_and_directory_isolation(mock_makedirs, mock_copytree, mock_mkdtemp, mock_chmod):
+@patch('shutil.rmtree', return_value=True)
+def test_process_isolation_and_directory_isolation(mock_makedirs, mock_copytree, mock_mkdtemp,
+                                                   mock_chmod, mock_rmtree):
+    def new_exists(path):
+        if path == "/project":
+            return False
+        return True
     rc = RunnerConfig('/')
     rc.artifact_dir = '/tmp/artifacts'
     rc.directory_isolation_path = '/tmp/dirisolation'
     rc.playbook = 'main.yaml'
     rc.command = 'ansible-playbook'
     rc.process_isolation = True
-    rc.prepare()
+    with patch('os.path.exists', new=new_exists):
+        rc.prepare()
 
     assert rc.command == [
         'bwrap',
+        '--die-with-parent',
         '--unshare-pid',
-        '--dev-bind', '/', '/',
+        '--dev', '/dev',
         '--proc', '/proc',
+        '--dir', '/tmp',
+        '--ro-bind', '/bin', '/bin',
+        '--ro-bind', '/etc', '/etc',
+        '--ro-bind', '/usr', '/usr',
+        '--ro-bind', '/opt', '/opt',
+        '--symlink', 'usr/lib64', '/lib64',
         '--bind', '/', '/',
-        '--chdir', '/tmp/dirisolation/foo',
+        '--chdir', os.path.realpath(rc.directory_isolation_path),
         'ansible-playbook', '-i', '/inventory', 'main.yaml',
     ]
 
@@ -447,31 +496,87 @@ def test_process_isolation_settings():
         path_exists.return_value=True
         rc.prepare()
 
-    assert rc.command[0:7] == [
+    expected = [
         'not_bwrap',
+        '--die-with-parent',
         '--unshare-pid',
-        '--dev-bind', '/', '/',
+        '--dev', '/dev',
         '--proc', '/proc',
+        '--dir', '/tmp',
+        '--ro-bind', '/bin', '/bin',
+        '--ro-bind', '/etc', '/etc',
+        '--ro-bind', '/usr', '/usr',
+        '--ro-bind', '/opt', '/opt',
+        '--symlink', 'usr/lib64', '/lib64',
     ]
+    index = len(expected)
+    assert rc.command[0:index] == expected
 
     # hide /home
-    assert rc.command[7] == '--bind'
-    assert 'ansible_runner_pi' in rc.command[8]
-    assert rc.command[9] == '/home'
+    assert rc.command[index] == '--bind'
+    assert 'ansible_runner_pi' in rc.command[index + 1]
+    assert rc.command[index + 2] == os.path.realpath('/home')  # needed for Mac
 
     # hide /var
-    assert rc.command[10] == '--bind'
-    assert 'ansible_runner_pi' in rc.command[11]
-    assert rc.command[12] == '/var' or rc.command[12] == '/private/var'
+    assert rc.command[index + 3] == '--bind'
+    assert 'ansible_runner_pi' in rc.command[index + 4]
+    assert rc.command[index + 5] in ('/var', '/private/var')
 
     # read-only bind
-    assert rc.command[13:16] == ['--ro-bind', '/venv', '/venv']
+    assert rc.command[index + 6:index + 9] == ['--ro-bind', '/venv', '/venv']
 
     # root bind
-    assert rc.command[16:19] == ['--bind', '/', '/']
+    assert rc.command[index + 9:index + 12] == ['--bind', '/', '/']
 
     # show /usr
-    assert rc.command[19:22] == ['--bind', '/usr', '/usr']
+    assert rc.command[index + 12:index + 15] == ['--bind', '/usr', '/usr']
 
     # chdir and ansible-playbook command
-    assert rc.command[22:] == ['--chdir', '/project', 'ansible-playbook', '-i', '/inventory', 'main.yaml']
+    assert rc.command[index + 15:] == ['--chdir', '/project', 'ansible-playbook', '-i', '/inventory', 'main.yaml']
+
+
+@patch('os.mkdir', return_value=True)
+def test_profiling_plugin_settings(mock_mkdir):
+    rc = RunnerConfig('/')
+    rc.playbook = 'main.yaml'
+    rc.command = 'ansible-playbook'
+    rc.resource_profiling = True
+    rc.resource_profiling_base_cgroup = 'ansible-runner'
+    rc.prepare()
+
+    expected_command_start = [
+        'cgexec',
+        '--sticky',
+        '-g',
+        'cpuacct,memory,pids:ansible-runner/{}'.format(rc.ident),
+        'ansible-playbook'
+    ]
+    for index, element in enumerate(expected_command_start):
+        assert rc.command[index] == element
+    assert 'main.yaml' in rc.command
+    assert rc.env['ANSIBLE_CALLBACK_WHITELIST'] == 'cgroup_perf_recap'
+    assert rc.env['CGROUP_CONTROL_GROUP'] == 'ansible-runner/{}'.format(rc.ident)
+    assert rc.env['CGROUP_OUTPUT_DIR'] == os.path.normpath(os.path.join(rc.private_data_dir, 'profiling_data'))
+    assert rc.env['CGROUP_OUTPUT_FORMAT'] == 'json'
+    assert rc.env['CGROUP_CPU_POLL_INTERVAL'] == '0.25'
+    assert rc.env['CGROUP_MEMORY_POLL_INTERVAL'] == '0.25'
+    assert rc.env['CGROUP_PID_POLL_INTERVAL'] == '0.25'
+    assert rc.env['CGROUP_FILE_PER_TASK'] == 'True'
+    assert rc.env['CGROUP_WRITE_FILES'] == 'True'
+    assert rc.env['CGROUP_DISPLAY_RECAP'] == 'False'
+
+
+@patch('os.mkdir', return_value=True)
+def test_profiling_plugin_settings_with_custom_intervals(mock_mkdir):
+    rc = RunnerConfig('/')
+    rc.playbook = 'main.yaml'
+    rc.command = 'ansible-playbook'
+    rc.resource_profiling = True
+    rc.resource_profiling_base_cgroup = 'ansible-runner'
+    rc.resource_profiling_cpu_poll_interval = '.5'
+    rc.resource_profiling_memory_poll_interval = '.75'
+    rc.resource_profiling_pid_poll_interval = '1.5'
+    rc.prepare()
+    assert rc.env['CGROUP_CPU_POLL_INTERVAL'] == '.5'
+    assert rc.env['CGROUP_MEMORY_POLL_INTERVAL'] == '.75'
+    assert rc.env['CGROUP_PID_POLL_INTERVAL'] == '1.5'
